//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.


#include <boost/lexical_cast.hpp>
#include "parallel/MeshLevelData.h"
#include "Global.h"

namespace AMDiS {

  using namespace std;


  void MeshLevelData::init(std::set<int> &neighbourRanks)
  {
    levelRanks.resize(1);
    levelRanks[0].insert(-1);
    nLevel = 1;
    
    levelNeighbours.resize(1);
    levelNeighbours[0] = neighbourRanks;

    mpiComms.resize(1);
    mpiComms[0] = MPI::COMM_WORLD;
  }


  void MeshLevelData::addLevel(std::set<int> &ranksInDomain, int domainId)
  {
    FUNCNAME("MeshLevelData()::addLevel()");

    TEST_EXIT(nLevel >= 1)("Mesh level structure is not initialized()");
    TEST_EXIT(nLevel == 1)("Only 2 level are supported yet!\n");

    levelRanks.push_back(ranksInDomain);
    nLevel++;

    levelNeighbours.resize(2);    
    levelNeighbours[1].clear();
    for (std::set<int>::iterator it = levelNeighbours[0].begin(); 
	 it != levelNeighbours[0].end(); ++it)
      if (levelRanks[1].count(*it) == 0)
	levelNeighbours[1].insert(*it);

    mpiComms.resize(2);
    mpiComms[1] = mpiComms[0].Split(domainId, mpiComms[0].Get_rank());
  }


  void MeshLevelData::serialize(ostream &out)
  {
  }

  
  void MeshLevelData::deserialize(istream &in)
  {
  }


  void MeshLevelData::print()
  {
    FUNCNAME("MeshLevelData::print()");

    using boost::lexical_cast;

    MSG("Print mesh level structure with %d levels: \n", nLevel);

    for (int i = 0; i < nLevel; i++) {
      string ranks = "ranks in level " + lexical_cast<string>(i) + ":";
      for (std::set<int>::iterator it = levelRanks[i].begin(); 
	   it != levelRanks[i].end(); ++it)
	ranks += " " + lexical_cast<string>(*it);

      string neighbours = "neighbours in level " + lexical_cast<string>(i) + ": ";
      for (std::set<int>::iterator it = levelNeighbours[i].begin(); 
	   it != levelNeighbours[i].end(); ++it)
	neighbours += " " + lexical_cast<string>(*it);

      if (ranks.length() < 250)
	MSG("  %s\n", ranks.c_str());
      else
	MSG("  ranks string to long!\n");

      if (neighbours.length() < 250)
	MSG("%s\n", neighbours.c_str());
      else
	MSG("  neighbours string to long!\n");
    }
  }
}
