// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file MeshLevelData.h */

#ifndef AMDIS_MESH_LEVEL_DATA_H
#define AMDIS_MESH_LEVEL_DATA_H


#include <iostream>
#include <set>
#include <vector>
#include <mpi.h>
#include "Global.h"

namespace AMDiS {

  using namespace std;

  class MeshLevelData 
  {
  public:
    MeshLevelData()
      : nLevel(0)
    {}

    void init(std::set<int> &neighbourRanks);

    void addLevel(std::set<int> &ranksInDomain, int domainId);

    // Writes all data of this object to an output stream.
    void serialize(ostream &out);

    // Reads the object data from an input stream.
    void deserialize(istream &in);

    void print();

    std::set<int>& getLevelRanks(int level)
    {
      TEST_EXIT_DBG(level < nLevel)("Should not happen!\n");
      
      return levelRanks[level];
    }

    std::set<int>& getLevelNeighbours(int level)
    {
      TEST_EXIT_DBG(level < nLevel)("Should not happen!\n");
      
      return levelNeighbours[level];
    }

   int getLevelNumber()
   {
     return nLevel;
   }

  protected:
    int nLevel;

    vector<std::set<int> > levelRanks;
    
    vector<std::set<int> > levelNeighbours;

    vector<MPI::Intracomm> mpiComms;
  };

}

#endif
